@@ -1,6 +1,7 @@
 from datetime import datetime
 
-from sqlalchemy import Column, Numeric, Integer, String, ForeignKey, DateTime
+from sqlalchemy import (Column, Numeric, Integer, String, ForeignKey, DateTime,
+                        UniqueConstraint)
 from sqlalchemy.orm import relationship, backref
 from flask.ext.sqlalchemy import SQLAlchemy
 
@@ -15,6 +16,79 @@ class SerialiseMixin:
         return {c.name: getattr(self, c.name) for c in self.__table__.columns}
 
 
+class DataPoint(db.Model, SerialiseMixin):
+    __tablename__ = 'data_point'
+
+    id = Column(Integer, primary_key=True)
+    created_at = Column(DateTime, nullable=False, index=True)
+    value = Column(Numeric, nullable=False)
+
+    device_series_id = Column(
+        Integer, ForeignKey('device_series.id'), nullable=False)
+    device_series = relationship(
+        "DeviceSeries", order_by=created_at.desc(),
+        backref=backref('data_points'))
+
+    def __init__(self, device_series, value, created_at=None):
+        self.created_at = datetime.utcnow()
+        super().__init__()
+        self.device_series = device_series
+        self.value = value
+        self.created_at = created_at
+
+    @classmethod
+    def record(cls, series, device, value, created_at=None):
+        ds = DeviceSeries.get_or_create(device=device, series=series)
+        data_point = DataPoint(device_series=ds, value=value)
+        if created_at is None:
+            data_point.created_at = datetime.utcnow()
+        db.session.add(data_point)
+        return data_point
+
+    def __repr__(self):
+        return "<Data Point(%s, value=%s, created_at=%s)>" % (
+            self.device_series, self.value, self.created_at)
+
+
+class DeviceSeries(db.Model, SerialiseMixin):
+    __tablename__ = 'device_series'
+
+    id = Column(Integer, primary_key=True)
+    created_at = Column(DateTime, nullable=False)
+    series_id = Column(Integer, ForeignKey('series.id'), nullable=False)
+    device_id = Column(Integer, ForeignKey('device.id'), nullable=False)
+
+    device = relationship(
+        "Device", backref=backref('device_series'), lazy='joined')
+    series = relationship(
+        "Series", backref=backref('device_series'), lazy='joined')
+
+    __table_args__ = (
+        UniqueConstraint('series_id', 'device_id', name='_series_device_uc'),
+    )
+
+    def __init__(self, device, series):
+        super().__init__()
+        self.created_at = datetime.utcnow()
+        self.series = series
+        self.device = device
+
+    @property
+    def latest_reading(self):
+        return DataPoint.query\
+            .filter_by(device_series=self)\
+            .order_by(DataPoint.created_at.desc())\
+            .limit(1).first()
+
+    @classmethod
+    def get_or_create(cls, **kwargs):
+        return get_or_create(cls, **kwargs)
+
+    def __repr__(self):
+        return "DeviceSeries(device_id=%s, series_id=%s)" % (
+            self.device_id, self.series_id)
+
+
 class Series(db.Model, SerialiseMixin):
     __tablename__ = 'series'
 
@@ -46,6 +120,15 @@ class Device(db.Model, SerialiseMixin):
     device_sub_type = Column(Integer)
     device_id = Column(String(20), nullable=False, unique=True)
 
+    series = relationship(
+        "Series",
+        secondary=("join(DeviceSeries, Series, DeviceSeries.series_id == "
+                   "Series.id)"),
+        primaryjoin=("and_(Device.id == DeviceSeries.device_id, Series.id == "
+                     "DeviceSeries.series_id)"),
+        secondaryjoin="Series.id == DeviceSeries.series_id"
+    )
+
     def __init__(self, device_type, device_sub_type, device_id, name=None):
         super().__init__()
         self.created_at = datetime.utcnow()
@@ -54,29 +137,6 @@ class Device(db.Model, SerialiseMixin):
         self.device_id = device_id
         self.name = name
 
-    def as_dict(self):
-
-        d = super().as_dict()
-
-        latest = {}
-
-        for series in self.get_series():
-
-            series_latest = (
-                DataPoint.query
-                .filter_by(device=self, series=series)
-                .order_by(DataPoint.created_at.desc())
-                .first()
-            )
-            latest[series.name] = SerialiseMixin.as_dict(series_latest)
-
-        d['latest'] = latest
-
-        return d
-
-    def get_series(self):
-        return Series.query.join(DataPoint).filter_by(device=self).all()
-
     @classmethod
     def get_or_create(cls, device_type, device_sub_type, device_id):
         return get_or_create(cls, device_type=device_type,
@@ -86,46 +146,3 @@ class Device(db.Model, SerialiseMixin):
     def __repr__(self):
         return "Device(name=%s, ID=%r)" % (
             self.name, self.device_id)
-
-
-class DataPoint(db.Model, SerialiseMixin):
-    __tablename__ = 'data_point'
-
-    id = Column(Integer, primary_key=True)
-    created_at = Column(DateTime, nullable=False)
-    value = Column(Numeric, nullable=False)
-    series_id = Column(Integer, ForeignKey('series.id'))
-    device_id = Column(Integer, ForeignKey('device.id'))
-
-    series = relationship("Series",
-                          backref=backref('data_points', order_by=id))
-    device = relationship("Device",
-                          backref=backref('data_points', order_by=id))
-
-    def __init__(self, series, device, value, created_at=None):
-        self.created_at = datetime.utcnow()
-        super().__init__()
-        self.series = series
-        self.device = device
-        self.value = value
-        self.created_at = created_at
-
-    @classmethod
-    def record(cls, series, device, value, created_at=None):
-        data_point = DataPoint(series=series, device=device, value=value)
-        if created_at is None:
-            data_point.created_at = datetime.utcnow()
-        db.session.add(data_point)
-        return data_point
-
-    def as_dict(self):
-
-        d = super().as_dict()
-        d['device'] = self.device.as_dict()
-        d['series'] = self.series.as_dict()
-
-        return d
-
-    def __repr__(self):
-        return "<Data Point(%s, %s, value=%s, created_at=%s)>" % (
-            self.series, self.device, self.value, self.created_at)
